#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# Copyright (c) 2011, Berkinet, (AKA Richard Perlman) All rights reserved.


################################################################################
import indigo  # Not required. But, removes lint errors
from berkinet import setLogLevel, logger
from indigoPluginUpdateChecker import updateChecker
import inspect
import threading
import time

################################################################################
# Globals
################################################################################
class ResettableTimer(threading.Thread):
	# Thanks to: http://socialwire.ca/2010/01/python-resettable-timer-example/
	def __init__(self, plugin, maxtime, actionType, actionId, actionCond, trigger):
		self.plugin = plugin

		if self.plugin.logLevel >2: self.plugin.debugLog('Timer Thread Received: maxtime:%s, actionType:%s, actionId:%s, actionCond:%s, trigger:%s' % (maxtime, actionType, actionId, actionCond, trigger))
		self.maxtime = maxtime
		self.actionType = actionType
		self.actionId = actionId
		self.actionCond = actionCond
		self.trigger = trigger
		self.inc = maxtime/2
		self.counter = 0
		self.active = True
		self.stop = False
		threading.Thread.__init__(self)
		self.setDaemon(True)
		
	def set_counter(self, t):
		self.counter = t
	
	def deactivate(self):
		self.active = False
	
	def kill(self):
		self.stop = True
	
	def reset(self):
		self.counter = 0
		self.active = True

	def run(self):
		while self.active:
			self.counter = 0
			while self.counter < self.maxtime:
				if self.plugin.logLevel >3: self.plugin.errorLog('Count:%s & active:%s & stop:%s' % (self.counter, self.active, self.stop))
				print self.counter
				self.counter += self.inc
				time.sleep(self.inc)
				if self.stop:
					self.plugin.errorLog('we were stopped')
					return
			
			if self.plugin.logLevel >2: self.plugin.errorLog('Finished count loop, active:%s & stop:%s' % (self.active, self.stop))
						
			if self.actionType == 'trigger':
				indigo.trigger.execute(self.actionId, ignoreConditions=self.actionCond)
			else:
				indigo.actionGroup.execute(self.actionId)
		
			del self.plugin.timerDict[self.trigger]
			self.active = False
			self.stop = True

		if self.plugin.logLevel >2: self.plugin.errorLog('Finished timer function')


################################################################################
class Plugin(indigo.PluginBase):
	########################################
	def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
		indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
	
		(self.logLevel, self.debug) = setLogLevel(self, pluginDisplayName, pluginPrefs.get("showDebugInfo1", "1"))       	
		self.log = logger(self)
		self.logName = pluginDisplayName
		self.pluginDisplayName = pluginDisplayName
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)

		self.updater = updateChecker(self, pluginId)
		self.updater.checkVersionPoll()

		self.groupDeviceOnDict = {}
		self.groupDeviceOffDict = {}
		self.groupCmdOnDict = {}
		self.groupCmdOffDict = {}
		self.groupVariableOnDict = {}
		self.groupVariableOffDict = {}
		self.triggersDict = {}
		self.timerDict = {}
		self.insteonAddressDict = {}
		self.custTrue = pluginPrefs.get("custTrue")
		self.custFalse = pluginPrefs.get("custFalse")
		self.log.log(2, dbFlg, "%s: Group Trigger Plugin Initialized" % (funcName), self.logName)
	
	########################################
   	def __del__(self):
		indigo.PluginBase.__del__(self)


	########################################
	# Start, Stop and Config changes
	########################################
	def startup(self):
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)

		indigo.devices.subscribeToChanges()
		indigo.variables.subscribeToChanges()
		indigo.insteon.subscribeToIncoming()
		# indigo.x10.subscribeToIncoming()
		self.getInsteonNameDict()

		self.log.log(2, dbFlg, "%s: Complete" % (funcName), self.logName)

	########################################
	def shutdown(self):
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)

		
	########################################
	def closedPrefsConfigUi (self, valuesDict, UserCancelled):
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)

		if UserCancelled is False:
			(self.logLevel, self.debug) = setLogLevel(self, self.pluginDisplayName, self.pluginPrefs.get("showDebugInfo1", "1"))       	
				
			self.custTrue = valuesDict["custTrue"]
			self.custFalse = valuesDict["custFalse"]

			self.log.log(2, dbFlg, "%s: Plugin config has been updated." % (funcName), self.logName)
	

	########################################
	# Trigger handlers
	########################################
	def triggerStartProcessing(self, trigger):
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)

		self.log.log(3, dbFlg, "%s: Plugin processing enabled for group trigger:%s" % (funcName, trigger.name), self.logName)
		self.log.log(4, dbFlg, "%s: Received trigger:%s" % (funcName, trigger), self.logName)

		if trigger.pluginProps['metaDevOnResponse'] == '0':
			actionOn = ""
			responseOn = ""
		elif trigger.pluginProps['metaDevOnResponse'] == '1':
			actionOn = trigger.id
			responseOn = 'trigger'
		elif trigger.pluginProps['metaDevOnResponse'] == '2':
			actionOn = trigger.pluginProps['metaDevOnTrigger']
			responseOn = 'trigger'
		elif trigger.pluginProps['metaDevOnResponse'] == '3':
			actionOn = trigger.pluginProps['metaDevOnAction']
			responseOn = 'action'

		if trigger.pluginProps['metaDevOffResponse'] == '0':
			actionOff = ""
			responseOff = ""
		elif trigger.pluginProps['metaDevOffResponse'] == '1':
			actionOff = trigger.id		
			responseOff = 'trigger'
		elif trigger.pluginProps['metaDevOffResponse'] == '2':
			actionOff = trigger.pluginProps['metaDevOffTrigger']					
			responseOff = 'trigger'
		elif trigger.pluginProps['metaDevOffResponse'] == '3':
			actionOff = trigger.pluginProps['metaDevOffAction']		
			responseOff = 'action'
			
		if trigger.pluginProps['metaDevOffSource'] == '0':
			selfOff = 'No'
		elif trigger.pluginProps['metaDevOffSource'] == '1':		
			selfOff = 'timerReset'
		elif trigger.pluginProps['metaDevOffSource'] == '2':		
			selfOff = 'timerFixed'
				
		timeOut = trigger.pluginProps['metaDevTimeOut']
		condOn = trigger.pluginProps['metaDevOnTriggerConditions']
		condOff = trigger.pluginProps['metaDevOffTriggerConditions']
		saveVarON = ""
		if trigger.pluginProps['saveToVarON']:
			saveVarON = trigger.pluginProps['varNameON']		
		saveVarOFF = ""
		if trigger.pluginProps['saveToVarOFF']:
			saveVarOFF = trigger.pluginProps['varNameOFF']

		self.triggersDict[trigger.id] = {'actionOn':actionOn, 'actionOff':actionOff,\
			'responseOn':responseOn, 'responseOff':responseOff, 'condOn':condOn,\
			'condOff':condOff, 'process':True, 'selfOff':selfOff, 'timeOut':timeOut,\
			'saveVarON':saveVarON, 'saveVarOFF':saveVarOFF}

		devList = []
		for devId in  trigger.pluginProps['metaDevOn']:	
			if self.logLevel > 3: self.debugLog('Device: Action on loop read:%s' % (devId))
			devList.append(devId)
		self.groupDeviceOnDict[trigger.id] = devList
		
		devList = []
		for devId in  trigger.pluginProps['metaDevOff']:
			if self.logLevel > 3: self.debugLog('Device: Action off loop read:%s' % (devId))
			devList.append(devId)
		self.groupDeviceOffDict[trigger.id] = devList
		self.log.log(3, dbFlg, "%s: Finished device processing:\n\tOn dict is: %s\n\tOff dict is: %s\n\tTriggers are: %s" % (funcName, self.groupDeviceOnDict, self.groupDeviceOffDict, self.triggersDict), self.logName)

		cmdList = []
		for devAddress in  trigger.pluginProps['metaCmdOn']:	
			devName = indigo.devices[int(devId)].name
			cmdList.append(devName)
		self.groupCmdOnDict[trigger.id] = cmdList
		
		cmdList = []
		for devName in  trigger.pluginProps['metaCmdOff']:	
			devName = indigo.devices[int(devId)].name
			cmdList.append(devName)
		self.groupCmdOffDict[trigger.id] = cmdList
		self.log.log(3, dbFlg, "%s: Finished command processing:\n\tOn dict is: %s\n\tOff dict is: %s\n\tTriggers are: %s" % (funcName, self.groupCmdOnDict, self.groupCmdOffDict, self.triggersDict), self.logName)
		
		varList = []
		for varId in  trigger.pluginProps['metaVarOn']:	
			if self.logLevel > 3: self.debugLog('Variable: Action on loop read:%s' % (varId))
			varList.append(varId)
		self.groupVariableOnDict[trigger.id] = varList
		
		varList = []
		for varId in  trigger.pluginProps['metaVarOff']:
			if self.logLevel > 3: self.debugLog('Variable: Action off loop read:%s' % (varId))
			varList.append(varId)
		self.groupVariableOffDict[trigger.id] = varList		
		self.log.log(3, dbFlg, "%s: Finished variable processing:\n\tOn dict is: %s\n\tOff dict is: %s\n\tTriggers are: %s" % (funcName, self.groupVariableOnDict, self.groupVariableOffDict, self.triggersDict), self.logName)

	
	########################################	
	def triggerStopProcessing(self, trigger):
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)

		# self.debugLog(u"<<-- entering deviceStopComm: %s, %d" % (trigger.name, trigger.id))
		if self.logLevel > 1: indigo.server.log("Plugin processing disabled for group trigger " + trigger.name, type="Group Triggers")
		self.triggersDict[trigger.id]['process'] = False
	

	########################################
	# Methods for changes in Device states, Variable values and received commands
	########################################
	def deviceUpdated(self, origDev, newDev):
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)

		# if self.logLevel > 3: self.debugLog(u"deviceUpdated ORIG:\n%s\nEND" % str(origDev))
		# if self.logLevel > 3: self.debugLog(u"deviceUpdated NEW:\n%s\nEND" % str(newDev))
		# self.log.log(3, dbFlg, "%s: received old dev:\n%s\nnew dev:\n%s" % (funcName, str(origDev), str(newDev)), self.logName)

		devNumber = str(newDev.id)
		
		devState = 'null'
		validDevice = False
		try:
			if newDev.states['onOffState']:
				devState = True
			else:
				devState = False
			validDevice = True	
		except Exception, err:
			# if self.logLevel > 3: self.debugLog("newDev onState ERROR: %s for device:%s" % (str(err), newDev.name))
			self.log.log(3, dbFlg, "%s: newDev onState ERROR: %s for device:%s" % (funcName, str(err), newDev.name), self.logName)

		# Skip input where the device state is actually unchanged		
		try:
			if origDev.states['onOffState']:
				origDevState = True
			else:
				origDevState = False
			
		except Exception, err:
			# if self.logLevel > 3: self.debugLog("origDev onState ERROR: %s for device:%s" % (str(err), newDev.name))
			self.log.log(3, dbFlg, "%s: newDev onState ERROR: %s for device:%s" % (funcName, str(err), newDev.name), self.logName)

		# Hack to avoid multiple updates from the alarm plugin	
		if 'onOffState' in origDev.states:  # Make sure we have an on/off device
			if validDevice and len(newDev.pluginId) > 0 and (devState == origDev.states['onOffState']):		
				self.debugLog("custom plugin hack: new state:%s, old state:%s" % (devState, origDev.states['onOffState']))
				validDevice = False
		
			self.debugLog("onState of %s is %s, and device is:%s \n" % (newDev.name, devState, validDevice))
					
			if validDevice:
				if devState != origDevState:	
					self.debugLog(u"Passed valid device test:%s" % newDev.name)
					for trigger in self.triggersDict:
						if self.triggersDict[trigger]['process']:
							self.debugLog(u"Found trigger:%s" % newDev.name)
							if devState:
								action = self.triggersDict[trigger]['actionOn']
								cond = self.triggersDict[trigger]['condOn']
								response = self.triggersDict[trigger]['responseOn']
								saveVarON = self.triggersDict[trigger]['saveVarON']
								selfOff = self.triggersDict[trigger]['selfOff']
				
								
								for tDevice in self.groupDeviceOnDict[trigger]:
									if self.logLevel > 3: self.debugLog("ON trigger: %s, device:%s, action:%s" % (trigger, tDevice, action))
									if tDevice == devNumber and len(str(action)) > 0:
										if len(saveVarON) > 0:
											try:
												indigo.variable.updateValue(saveVarON, newDev.name)
											except:
												self.errorLog('Device name could not be saved to Indigo variable "%s". Does it exist?' % saveVarON)
										if self.logLevel > 2: self.debugLog('tDevice:%s, devNumber:%s' % (tDevice, devNumber))
										action = int(action)
										if trigger not in self.timerDict: # There is no auto-off in progress
											if response == 'trigger':
												indigo.trigger.execute(action, ignoreConditions=cond)
											else:
												indigo.actionGroup.execute(action)
									
										if selfOff == 'timerReset' or selfOff == 'timerFixed':
											if self.logLevel > 2: self.debugLog("Device: Auto-Off: type:%s" % (selfOff))
											offAction = int(self.triggersDict[trigger]['actionOff'])
											offResponse = self.triggersDict[trigger]['responseOff']
											offCond = self.triggersDict[trigger]['condOff']
											offTimeOut = int(self.triggersDict[trigger]['timeOut'])
											if trigger in self.timerDict and selfOff == 'timerReset':
												self.timerDict[trigger].reset()
											elif trigger not in self.timerDict :
												self.timerDict[trigger] = ResettableTimer(self, offTimeOut, offResponse, offAction, offCond, trigger)
												self.timerDict[trigger].start() 
											else:
												self.errorLog('Impossible condition')										
			
							else:
								action = self.triggersDict[trigger]['actionOff']
								cond = self.triggersDict[trigger]['condOff']
								response = self.triggersDict[trigger]['responseOff']
								saveVarOFF = self.triggersDict[trigger]['saveVarOFF']
						
								# Check to see is auto-off is set
								if self.triggersDict[trigger]['selfOff'] == 'No':
									for tDevice in self.groupDeviceOffDict[trigger]:
										if self.logLevel > 3: self.debugLog("OFF trigger: %s, device:%s, action:%s" % (trigger, tDevice, action))
										if tDevice == devNumber and len(str(action)) > 0:
											if len(saveVarOFF) > 0:
												try:
													indigo.variable.updateValue(saveVarOFF, newDev.name)
												except:
													self.errorLog('Device name could not be saved to Indigo variable "%s". Does it exist?' % saveVarOFF)
													
											action = int(action)
											if response == 'trigger':
												indigo.trigger.execute(action, ignoreConditions=cond)
											else:
												indigo.actionGroup.execute(action)	
		self.log.log(2, dbFlg, "%s: Complete." % (funcName), self.logName)
								
	########################################
	def variableUpdated(self, origVar, newVar):
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)

		# self.debugLog(u"deviceUpdated orig: \n" + str(origVar))
		# self.debugLog(u"Variable Updated new: \n" + str(newVar))

		varNumber = str(newVar.id)
		#self.debugLog("Variable update received:%s" % (varNumber))
		
		varValue = newVar.value.lower()
		self.debugLog('Var value in: %s, lower: :%s:' % (newVar.value, varValue))
		
		# Normalize the varValue readings to true/false
		if varValue == 'on':
			varValue = 'true'
		elif varValue == 'off':
			varValue = 'false'
		elif varValue == 'yes':
			varValue = 'true'
		elif varValue == 'no':
			varValue = 'false'
		elif varValue == 'closed':
			varValue = 'true'
		elif varValue == 'open':
			varValue = 'false'
		elif varValue == self.custTrue:
			varValue = 'true'
		elif varValue == self.custFalse:
			varValue = 'false'
		elif varValue == '1':
			varValue = 'true'
		elif varValue == '0':
			varValue = 'false'
		else:
			try:
				if float(varValue) != 0:
					varValue = 'true'
			except:
				pass
		
		for trigger in self.triggersDict:
			if varNumber in self.groupVariableOnDict[trigger] or varNumber in self.groupVariableOffDict[trigger]:
				if self.logLevel > 3: self.debugLog('In outer trigger loop. Trigger: %s, varValue: :%s:' % (trigger, varValue))
				if self.triggersDict[trigger]['process']:
					self.debugLog('In inner trigger loop. trigger def is: %s, varValue: :%s:' % (self.triggersDict[trigger], varValue))
					if varValue == 'true':
						self.debugLog('Entered Variable true process section')
						action = self.triggersDict[trigger]['actionOn']
						cond = self.triggersDict[trigger]['condOn']
						response = self.triggersDict[trigger]['responseOn']
						selfOff = self.triggersDict[trigger]['selfOff']
						saveVarON = self.triggersDict[trigger]['saveVarON']
									
						for tVariable in self.groupVariableOnDict[trigger]:
							if self.logLevel > 3: self.debugLog("trigger on false: %s, Var reporting:%s, trigger member:%s, action:%s" % (trigger, varNumber, tVariable, action))
							if tVariable == varNumber and len(str(action)) > 0:
								if len(saveVarON) > 0:
									try:
										indigo.variable.updateValue(saveVarON, newVar.name)
									except:
										self.errorLog('Variable name could not be saved to Indigo variable "%s". Does it exist?' % saveVarON)
								if self.logLevel > 2: self.debugLog('tVariable:%s, varNumber:%s' % (tVariable, varNumber))								
								action = int(action)
								if trigger not in self.timerDict: # There is no auto-off in progress
									if response == 'trigger':
										indigo.trigger.execute(action, ignoreConditions=cond)
									else:
										indigo.actionGroup.execute(action)
									
								if selfOff == 'timerReset' or selfOff == 'timerFixed':
									if self.logLevel > 2: self.debugLog("Variable: Auto-Off: type:%s" % (selfOff))
									offAction = int(self.triggersDict[trigger]['actionOff'])
									offResponse = self.triggersDict[trigger]['responseOff']
									offCond = self.triggersDict[trigger]['condOff']
									offTimeOut = int(self.triggersDict[trigger]['timeOut'])
									if trigger in self.timerDict and selfOff == 'timerReset':
										self.timerDict[trigger].reset()
									elif trigger not in self.timerDict :
										self.timerDict[trigger] = ResettableTimer(self, offTimeOut, offResponse, offAction, offCond, trigger)
										self.timerDict[trigger].start() 
									else:
										self.errorLog('Impossible condition')				
							
					elif varValue == 'false':
						self.debugLog('Entered Variable false process section')
						action = self.triggersDict[trigger]['actionOff']
						cond = self.triggersDict[trigger]['condOff']
						response = self.triggersDict[trigger]['responseOff']
						saveVarOFF = self.triggersDict[trigger]['saveVarOFF']

						# Check to see is auto-off is set
						if self.triggersDict[trigger]['selfOff'] == 'No':
							self.debugLog('False var val, no timer')
							for tVariable in self.groupVariableOffDict[trigger]:
								if self.logLevel > 3: self.debugLog("trigger on false. Read item for var: %s, Var reporting:%s, action:%s" % (tVariable, varNumber, action))
								if tVariable == varNumber and len(str(action)) > 0:
									if len(saveVarOFF) > 0:
										try:
											indigo.variable.updateValue(saveVarOFF, newVar.name)
										except:
											self.errorLog('Variable name could not be saved to Indigo variable "%s". Does it exist?' % saveVarOFF)
									
									action = int(action)
									if response == 'trigger':
										indigo.trigger.execute(action, ignoreConditions=cond)
									else:
										indigo.actionGroup.execute(action)	
					else:
						self.errorLog(u'Variable value "%s" is neither True of False' % (varValue))	
		self.log.log(2, dbFlg, "%s: Complete." % (funcName), self.logName)
					
	########################################
	def insteonCommandReceived(self, cmd):
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)

		# self.debugLog(u"Insteon command received: \n" + str(cmd))

		cmdTxt = cmd.cmdFunc
		# Fnd the Indigo device name assosciated with this Insteon address
		try:
			cmdDev = self.insteonAddressDict[cmd.address]
		except:
			self.errorLog(u"no address to name data found for received command:%s" % cmd)

		self.debugLog(u"Insteon command received from device:%s-%s" % (cmdDev, cmdTxt))

		# Only continue if we are dealing with on and off commands
		if cmdTxt == 'on' or cmdTxt == 'off':
			for trigger in self.triggersDict:
				if cmdDev in self.groupCmdOnDict[trigger]:
					if self.logLevel > 3: self.debugLog('In outer trigger loop. Trigger: %s, varValue: :%s:' % (trigger, cmdTxt))
					if self.triggersDict[trigger]['process']:
						self.debugLog('In inner trigger loop. trigger def is: %s, varValue: :%s:' % (self.triggersDict[trigger], cmdTxt))
						if cmdTxt == 'on':
							self.debugLog('Entered Command "on" process section')
							action = self.triggersDict[trigger]['actionOn']
							cond = self.triggersDict[trigger]['condOn']
							response = self.triggersDict[trigger]['responseOn']
							selfOff = self.triggersDict[trigger]['selfOff']
							saveVarON = self.triggersDict[trigger]['saveVarON']
										
							for tCmd in self.groupCmdOnDict[trigger]:
								if self.logLevel > 3: self.debugLog("trigger on false: %s, Var reporting:%s, trigger member:%s, action:%s" % (trigger, cmdDev, tCmd, action))
								if tCmd == cmdDev and len(str(action)) > 0:
									if len(saveVarON) > 0:
										try:
											indigo.variable.updateValue(saveVarON, cmdDev)
										except:
											self.errorLog('Variable name could not be saved to Indigo variable "%s". Does it exist?' % saveVarON)
									if self.logLevel > 2: self.debugLog('tCmd:%s, cmdDev:%s' % (tCmd, cmdDev))								
									action = int(action)
									if trigger not in self.timerDict: # There is no auto-off in progress
										if response == 'trigger':
											indigo.trigger.execute(action, ignoreConditions=cond)
										else:
											indigo.actionGroup.execute(action)
										
									if selfOff == 'timerReset' or selfOff == 'timerFixed':
										if self.logLevel > 2: self.debugLog("Variable: Auto-Off: type:%s" % (selfOff))
										offAction = int(self.triggersDict[trigger]['actionOff'])
										offResponse = self.triggersDict[trigger]['responseOff']
										offCond = self.triggersDict[trigger]['condOff']
										offTimeOut = int(self.triggersDict[trigger]['timeOut'])
										if trigger in self.timerDict and selfOff == 'timerReset':
											self.timerDict[trigger].reset()
										elif trigger not in self.timerDict :
											self.timerDict[trigger] = ResettableTimer(self, offTimeOut, offResponse, offAction, offCond, trigger)
											self.timerDict[trigger].start() 
										else:
											self.errorLog('Impossible condition')				
								
						elif cmdTxt == 'off':
							self.debugLog('Entered command "off" process section')
							action = self.triggersDict[trigger]['actionOff']
							cond = self.triggersDict[trigger]['condOff']
							response = self.triggersDict[trigger]['responseOff']
							saveVarOFF = self.triggersDict[trigger]['saveVarOFF']

							# Check to see is auto-off is set
							if self.triggersDict[trigger]['selfOff'] == 'No':
								self.debugLog('False var val, no timer')
								for tCmd in self.groupCmdOffDict[trigger]:
									if self.logLevel > 3: self.debugLog("trigger on false. Read item for var: %s, cmd dev:%s, action:%s" % (tCmd, cmdDev, action))
									if tCmd == cmdDev and len(str(action)) > 0:
										if len(saveVarOFF) > 0:
											try:
												indigo.variable.updateValue(saveVarOFF, cmdDev)
											except:
												self.errorLog('Variable name could not be saved to Indigo variable "%s". Does it exist?' % saveVarOFF)
										
										action = int(action)
										if response == 'trigger':
											indigo.trigger.execute(action, ignoreConditions=cond)
										else:
											indigo.actionGroup.execute(action)	
					else:
						if self.logLevel > 3: self.debugLog(u'Command value "%s" is neither on nor off' % (cmdTxt))	

		self.log.log(2, dbFlg, "%s: Complete." % (funcName), self.logName)

	########################################
	def x10CommandReceived(self, cmd):
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)

	  # self.debugLog(u"x10CommandReceived: \n" + str(cmd))
	  # self.insteonCommandReceived(cmd)


	########################################	
	# ConfigUI supporting methods
	########################################		
	def getDeviceList(self, filter="", valuesDict=None, typeId="", targetId=0):
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)

		myArray = []
		
		for dev in indigo.devices:
			try:
				 if 'onOffState' in dev.states or 'displayState' in dev.states:
					myArray.append((dev.id, dev.name))
				 	self.log.log(3, dbFlg, "%s: found device:%s id:%s" % (funcName, str(dev.name), str(dev.id)), self.logName)
			except:
				pass

		self.log.log(2, dbFlg, "%s: Finished creating device list:\n\t%s" % (funcName, myArray), self.logName)

		return myArray

	########################################
	def getVariableList(self, filter="", valuesDict=None, typeId="", targetId=0):
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)

		myArray = []
		
		for var in indigo.variables:
			try:
				varValue = var.value.lower()
				# Normalize the varValue readings to true/false
				if varValue == 'on':
					varValue = 'true'
				elif varValue == 'off':
					varValue = 'false'
				elif varValue == 'yes':
					varValue = 'true'
				elif varValue == 'no':
					varValue = 'false'
				elif varValue == 'closed':
					varValue = 'true'
				elif varValue == 'open':
					varValue = 'false'
				elif varValue == self.custTrue:
					varValue = 'true'
				elif varValue == self.custFalse:
					varValue = 'false'
				elif varValue == '1':
					varValue = 'true'
				elif varValue == '0':
					varValue = 'false'
				else:
					try:
						if float(varValue) != 0:
							varValue = 'true'
					except:
						pass
				if varValue == 'true' or varValue == 'false':
					myArray.append((var.id, var.name))
			except:
				pass

		self.log.log(2, dbFlg, "%s: Finished creating variable list:\n\t%s" % (funcName, myArray), self.logName)

		return myArray


	########################################	
	# Misc supporting methods
	########################################		
	def getInsteonNameDict(self): # Create an Insteon Address to Indigo device name lookup
		funcName = inspect.stack()[0][3]
		dbFlg = False
		self.log.log(2, dbFlg, "%s Called" % (funcName), self.logName)
		
		for dev in indigo.devices.iter("indigo.insteon"):
			self.insteonAddressDict[str(dev.address)] = str(dev.name)
		 	self.log.log(3, dbFlg, "%s: found Insteon device name:%s address:%s" % (funcName, str(dev.name), str(dev.address)), self.logName)

		self.log.log(2, dbFlg, "%s: Finished creating Insteon Address dict:\n\t%s" % (funcName, self.insteonAddressDict), self.logName)

		return
	

